class Research
  include Mongoid::Document
  include Mongoid::Timestamps
  include OrbitModel::Status
  include Slug

  belongs_to :member_profile

  field :research_title, as: :slug_title, localize: true
  field :authors, localize: true
  field :extracted_chapters, localize: true
  
  field :year
  field :language
  field :publish_date , :type => Date
  field :keywords
  field :url
  field :note
  field :rss2_id
  field :create_user_id, :type => BSON::ObjectId
  field :update_user_id, :type => BSON::ObjectId
  
  paginates_per 10
  
  has_many :research_files, :autosave => true, :dependent => :destroy

  belongs_to :research_category
  
  accepts_nested_attributes_for :research_files, :allow_destroy => true

  before_validation :add_http

  scope :sort_for_frontend, ->{ where(:is_hidden=>false).order_by(:year => "desc", :publish_date => "desc") }

  def get_plugin_data(fields_to_show)
    plugin_datas = []
    fields_to_show.each do |field|
      plugin_data = self.get_plugin_field_data(field) rescue nil
      next if plugin_data.blank? or plugin_data['value'].blank?
      plugin_datas << plugin_data
    end
    plugin_datas
  end

  def self.get_plugin_datas_to_member(datas)

    fields_to_show = [
      "year",
      "research_category",
      "research_title",
      "publish_date",
      "authors"
    ]

    fields_to_remove = []

    pd_title = []

    fields_to_show.each do |t|
      if (self.fields[t].type.to_s == "String" || self.fields[t].type.to_s == "Object" rescue false)
        fields_to_remove << t if (datas.where(t.to_sym.ne => nil, t.to_sym.ne => "").count == 0 rescue false)
      else
        fields_to_remove << t if (datas.where(t.to_sym.ne => nil).count == 0 rescue false)
      end
      pd_title <<  {
          "plugin_data_title" => I18n.t("personal_research.#{t}")
        } if !fields_to_remove.include?(t)
    end

    fields_to_show = fields_to_show - fields_to_remove

    plugin_datas = datas.sort_for_frontend.collect.with_index do |p, index|

      pd_data = []
      fields_to_show.collect do |t|
        if t == "research_title"
          pd_data << { "data_title" => "<a href='#{OrbitHelper.url_to_plugin_show(p.to_param,'personal_research')}'>#{p.send(t)}" }
        elsif t == "research_category"
          pd_data << {"data_title" => (p.research_category.title rescue "")}
        else
          pd_data << { "data_title" => p.send(t) }
        end
      end

      {
        "pd_datas" => pd_data,
        "type-sort" => (p.research_category.sort_position.to_i rescue -1),
        "sort-index" => index
      }

    end
    plugin_datas = plugin_datas.sort_by{|pd| [pd["type-sort"], pd["sort-index"]]}
    return [pd_title,plugin_datas]

  end

  def get_plugin_field_data(field)
    case field
    when "language"
      value =  self.language.nil? ? "" : I18n.t(self.language) rescue ""
    when "publish_date"
      value = self.publish_date.strftime("%Y-%m") rescue ""
    when "research_category"
      value = self.research_category.title rescue ""
    when "file"
      files = []
      self.research_files.each do |research_file|  
        url = research_file.file.url
        title = (research_file.title.blank? ? File.basename(research_file.file.path) : research_file.title)
        files << "<li><a href='#{url}' target='_blank'>#{title}</li>"
      end
      value = files.join("")
    else
      value = self.send(field) rescue ""
    end

    value = (value =~ /\A#{URI::regexp(['http', 'https'])}\z/) ? "<a href='#{value}' target='blank'>#{value}</a>" : value
    
    {
      "key"=>field,
      "title_class"=>"research-#{field.gsub('_','-')}-field",
      "value_class"=>"research-#{field.gsub('_','-')}-value",
      "title"=>I18n.t('personal_research.'+field),
      "value"=>value
    }
  end

  protected

  def add_http
    unless self.url.blank? || self.url[/^http:\/\//] || self.url[/^https:\/\//]
      self.url = 'http://' + self.url
    end
  end
  
end